@@ -1,25 +1,26 @@
 /*
-* TCP_server.c -- Implementation of the TCP relay server part of Tox.
-*
-*  Copyright (C) 2014 Tox project All Rights Reserved.
-*
-*  This file is part of Tox.
-*
-*  Tox is free software: you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation, either version 3 of the License, or
-*  (at your option) any later version.
-*
-*  Tox is distributed in the hope that it will be useful,
-*  but WITHOUT ANY WARRANTY; without even the implied warranty of
-*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*  GNU General Public License for more details.
-*
-*  You should have received a copy of the GNU General Public License
-*  along with Tox.  If not, see <http://www.gnu.org/licenses/>.
-*
-*/
+ * Implementation of the TCP relay server part of Tox.
+ */
 
+/*
+ * Copyright © 2016-2017 The TokTok team.
+ * Copyright © 2014 Tox project.
+ *
+ * This file is part of Tox, the free peer to peer instant messenger.
+ *
+ * Tox is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Tox is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Tox.  If not, see <http://www.gnu.org/licenses/>.
+ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -39,7 +40,7 @@ struct TCP_Server {
     int efd;
     uint64_t last_run_pinged;
 #endif
-    sock_t *socks_listening;
+    Socket *socks_listening;
     unsigned int num_listening_socks;
 
     uint8_t public_key[CRYPTO_PUBLIC_KEY_SIZE];
@@ -68,10 +69,16 @@ size_t tcp_server_listen_count(const TCP_Server *tcp_server)
     return tcp_server->num_listening_socks;
 }
 
+/* this is needed to compile on Android below API 21
+*/
+#ifndef EPOLLRDHUP
+#define EPOLLRDHUP 0x2000
+#endif
+
 /* return 1 on success
  * return 0 on failure
  */
-static int bind_to_port(sock_t sock, int family, uint16_t port)
+static int bind_to_port(Socket sock, int family, uint16_t port)
 {
     struct sockaddr_storage addr = {0};
     size_t addrsize;
@@ -225,7 +232,7 @@ static int del_accepted(TCP_Server *TCP_server, int index)
 /* return the amount of data in the tcp recv buffer.
  * return 0 on failure.
  */
-unsigned int TCP_socket_data_recv_buffer(sock_t sock)
+unsigned int TCP_socket_data_recv_buffer(Socket sock)
 {
 #if defined(_WIN32) || defined(__WIN32__) || defined (WIN32)
     unsigned long count = 0;
@@ -245,7 +252,7 @@ unsigned int TCP_socket_data_recv_buffer(sock_t sock)
  * return 0 if nothing has been read from socket.
  * return ~0 on failure.
  */
-uint16_t read_TCP_length(sock_t sock)
+uint16_t read_TCP_length(Socket sock)
 {
     unsigned int count = TCP_socket_data_recv_buffer(sock);
 
@@ -275,7 +282,7 @@ uint16_t read_TCP_length(sock_t sock)
  * return length on success
  * return -1 on failure/no data in buffer.
  */
-int read_TCP_packet(sock_t sock, uint8_t *data, uint16_t length)
+int read_TCP_packet(Socket sock, uint8_t *data, uint16_t length)
 {
     unsigned int count = TCP_socket_data_recv_buffer(sock);
 
@@ -297,7 +304,7 @@ int read_TCP_packet(sock_t sock, uint8_t *data, uint16_t length)
  * return 0 if could not read any packet.
  * return -1 on failure (connection must be killed).
  */
-int read_packet_TCP_secure_connection(sock_t sock, uint16_t *next_packet_length, const uint8_t *shared_key,
+int read_packet_TCP_secure_connection(Socket sock, uint16_t *next_packet_length, const uint8_t *shared_key,
                                       uint8_t *recv_nonce, uint8_t *data, uint16_t max_len)
 {
     if (*next_packet_length == 0) {
@@ -522,7 +529,7 @@ static int kill_accepted(TCP_Server *TCP_server, int index)
         rm_connection_index(TCP_server, &TCP_server->accepted_connection_array[index], i);
     }
 
-    sock_t sock = TCP_server->accepted_connection_array[index].sock;
+    Socket sock = TCP_server->accepted_connection_array[index].sock;
 
     if (del_accepted(TCP_server, index) != 0) {
         return -1;
@@ -964,7 +971,7 @@ static int confirm_TCP_connection(TCP_Server *TCP_server, TCP_Secure_Connection
 /* return index on success
  * return -1 on failure
  */
-static int accept_connection(TCP_Server *TCP_server, sock_t sock)
+static int accept_connection(TCP_Server *TCP_server, Socket sock)
 {
     if (!sock_valid(sock)) {
         return -1;
@@ -996,9 +1003,9 @@ static int accept_connection(TCP_Server *TCP_server, sock_t sock)
     return index;
 }
 
-static sock_t new_listening_TCP_socket(int family, uint16_t port)
+static Socket new_listening_TCP_socket(int family, uint16_t port)
 {
-    sock_t sock = socket(family, SOCK_STREAM, IPPROTO_TCP);
+    Socket sock = socket(family, SOCK_STREAM, IPPROTO_TCP);
 
     if (!sock_valid(sock)) {
         return ~0;
@@ -1041,7 +1048,7 @@ TCP_Server *new_TCP_server(uint8_t ipv6_enabled, uint16_t num_sockets, const uin
         return NULL;
     }
 
-    temp->socks_listening = (sock_t *)calloc(num_sockets, sizeof(sock_t));
+    temp->socks_listening = (Socket *)calloc(num_sockets, sizeof(Socket));
 
     if (temp->socks_listening == NULL) {
         free(temp);
@@ -1073,7 +1080,7 @@ TCP_Server *new_TCP_server(uint8_t ipv6_enabled, uint16_t num_sockets, const uin
 #endif
 
     for (i = 0; i < num_sockets; ++i) {
-        sock_t sock = new_listening_TCP_socket(family, ports[i]);
+        Socket sock = new_listening_TCP_socket(family, ports[i]);
 
         if (sock_valid(sock)) {
 #ifdef TCP_SERVER_USE_EPOLL
@@ -1117,7 +1124,7 @@ static void do_TCP_accept_new(TCP_Server *TCP_server)
     for (i = 0; i < TCP_server->num_listening_socks; ++i) {
         struct sockaddr_storage addr;
         socklen_t addrlen = sizeof(addr);
-        sock_t sock;
+        Socket sock;
 
         do {
             sock = accept(TCP_server->socks_listening[i], (struct sockaddr *)&addr, &addrlen);
@@ -1285,7 +1292,7 @@ static void do_TCP_epoll(TCP_Server *TCP_server)
         int n;
 
         for (n = 0; n < nfds; ++n) {
-            sock_t sock = events[n].data.u64 & 0xFFFFFFFF;
+            Socket sock = events[n].data.u64 & 0xFFFFFFFF;
             int status = (events[n].data.u64 >> 32) & 0xFF, index = (events[n].data.u64 >> 40);
 
             if ((events[n].events & EPOLLERR) || (events[n].events & EPOLLHUP) || (events[n].events & EPOLLRDHUP)) {
@@ -1326,7 +1333,7 @@ static void do_TCP_epoll(TCP_Server *TCP_server)
                     socklen_t addrlen = sizeof(addr);
 
                     while (1) {
-                        sock_t sock_new = accept(sock, (struct sockaddr *)&addr, &addrlen);
+                        Socket sock_new = accept(sock, (struct sockaddr *)&addr, &addrlen);
 
                         if (!sock_valid(sock_new)) {
                             break;
